// Copyright 1998-2017 Epic Games, Inc. All Rights Reserved.

#include "Net/UnitTestNetDriver.h"
#include "Engine/Channel.h"

#include "Net/UnitTestNetConnection.h"
#include "Net/UnitTestChannel.h"

/**
 * Default constructor
 */
UUnitTestNetDriver::UUnitTestNetDriver(const FObjectInitializer& ObjectInitializer)
	: Super(ObjectInitializer)
{
}

bool UUnitTestNetDriver::InitConnectionClass()
{
	bool bReturnVal = false;

	if (NetConnectionClass == NULL)
	{
		NetConnectionClass = UUnitTestNetConnection::StaticClass();
		bReturnVal = true;
	}

	return bReturnVal;
}

/**
 * @todo #JohnBDoc
 */
bool UUnitTestNetDriver::InitConnect(FNetworkNotify* InNotify, const FURL& ConnectURL, FString& Error)
{
	bool bReturnVal = false;

	// Hack: Replace the control channel with a stripped down unit test channel
	UClass* OrigClass = UChannel::ChannelClasses[CHTYPE_Control];
	UChannel::ChannelClasses[CHTYPE_Control] = UUnitTestChannel::StaticClass();

	bReturnVal = Super::InitConnect(InNotify, ConnectURL, Error);

	UChannel::ChannelClasses[CHTYPE_Control] = OrigClass;

	return bReturnVal;
}

void UUnitTestNetDriver::ProcessRemoteFunction(AActor* Actor, UFunction* Function, void* Parameters, FOutParmRec* OutParms,
												FFrame* Stack, UObject* SubObject)
{
	bool bAllowRPC = true;

	if (SendRPCDel.IsBound())
	{
		bAllowRPC = SendRPCDel.Execute(Actor, Function, Parameters, OutParms, Stack, SubObject);
	}

	if (bAllowRPC)
	{
		Super::ProcessRemoteFunction(Actor, Function, Parameters, OutParms, Stack, SubObject);
	}
}




