// Copyright 1998-2017 Epic Games, Inc. All Rights Reserved.

#pragma once

#include "CoreMinimal.h"
#include "UObject/ObjectMacros.h"
#include "UObject/Object.h"
#include "GameCircleRuntimeSettings.generated.h"

/**
* Implements the settings for the OnlineSubsystemGameCircle plugin.
*/
UCLASS(config = Engine, defaultconfig)
class GAMECIRCLERUNTIMESETTINGS_API UGameCircleRuntimeSettings : public UObject
{
	GENERATED_UCLASS_BODY()

	// Enables the Amazon GameCircle OnlineSubsystem
	UPROPERTY(GlobalConfig, EditAnywhere, config, Category=Settings)
	bool bEnableAmazonGameCircleSupport;

	// Enables Amazon In App Purchasing
	UPROPERTY(GlobalConfig, EditAnywhere, config, Category=Settings)
	bool bSupportsInAppPurchasing;

	// Removes touchscreen hardware requirement when enabled
	UPROPERTY(GlobalConfig, EditAnywhere, config, Category=Settings, DisplayName="Enable Fire TV Support")
	bool bEnableFireTVSupport;

	// The path of the api_key text file generated by Amazon from your debug keystore from the Game Project directory
	UPROPERTY(GlobalConfig, EditAnywhere, config, Category = Settings, DisplayName="Debug API Key File")
	FString DebugAPIKeyFile;

	// The path of the api_key text file generated by Amazon from your distribution keystore from the Game Project directory
	UPROPERTY(GlobalConfig, EditAnywhere, config, Category = Settings, Displayname="Distribution API Key File")
	FString DistributionAPIKeyFile;

#if WITH_EDITOR
	// UObject interface
	virtual void PostEditChangeProperty(struct FPropertyChangedEvent& PropertyChangedEvent) override;
	virtual void PostInitProperties() override;
	// End of UObject interface
#endif
};
