// Copyright 1998-2017 Epic Games, Inc. All Rights Reserved.

#pragma once

#include "CoreMinimal.h"
#include "Templates/SubclassOf.h"
#include "Input/Reply.h"
#include "AssetData.h"
#include "Blueprint/UserWidget.h"
#include "Widgets/IToolTip.h"
#include "Templates/WidgetTemplateClass.h"

class UWidgetTree;

/**
* A template for classes generated by UWidgetBlueprint assets
* Accounts for the possibility that a the blueprint may not be loaded when the palette is created
*/
class FWidgetTemplateBlueprintClass : public FWidgetTemplateClass
{
public:
	/** 
	* Constructor
	* @param InWidgetAssetData The FAssetData for the widget blueprint asset
	* @param InUserWidgetClass The user-created widget class that will be created by this template
	*/
	FWidgetTemplateBlueprintClass(const FAssetData& InWidgetAssetData, TSubclassOf<UUserWidget> InUserWidgetClass = nullptr);

	/** Destructor */
	virtual ~FWidgetTemplateBlueprintClass();

	/** Gets the category for the widget */
	virtual FText GetCategory() const override;

	/** Creates an instance of the widget for the tree. */
	virtual UWidget* Create(UWidgetTree* Tree) override;

	/** The icon coming from the default object of the class */
	virtual const FSlateBrush* GetIcon() const override;

	/** Gets the tooltip widget for this palette item. */
	virtual TSharedRef<IToolTip> GetToolTip() const override;

	/** Opens the widget blueprint for edit */
	virtual FReply OnDoubleClicked() override;

	/** Gets the asset data for this widget blueprint */
	FAssetData GetWidgetAssetData();

	/** Returns true if the supplied class is supported by this template */
	static bool Supports(UClass* InClass);

protected:
	/** The asset data for the widget blueprint */
	FAssetData WidgetAssetData;
};
