/*
 * Copyright (c) 2008-2015, NVIDIA CORPORATION.  All rights reserved.
 *
 * NVIDIA CORPORATION and its licensors retain all intellectual property
 * and proprietary rights in and to this software, related documentation
 * and any modifications thereto.  Any use, reproduction, disclosure or
 * distribution of this software and related documentation without an express
 * license agreement from NVIDIA CORPORATION is strictly prohibited.
 */


#ifndef MODULE_CONVERSIONAPEXEMITTERASSETPARAMETERS_0P7_0P8H_H
#define MODULE_CONVERSIONAPEXEMITTERASSETPARAMETERS_0P7_0P8H_H

#include "NvParamConversionTemplate.h"
#include "ApexEmitterAssetParameters_0p7.h"
#include "ApexEmitterAssetParameters_0p8.h"

#include "nvparameterized/NvParamUtils.h"

#define PARAM_RET(x) if( (x) != NvParameterized::ERROR_NONE ) \
								{ PX_ASSERT(0 && "INVALID Parameter"); return false; }

namespace nvidia {
namespace apex {
namespace legacy {


typedef NvParameterized::ParamConversionTemplate<nvidia::parameterized::ApexEmitterAssetParameters_0p7, 
						nvidia::parameterized::ApexEmitterAssetParameters_0p8, 
						nvidia::parameterized::ApexEmitterAssetParameters_0p7::ClassVersion, 
						nvidia::parameterized::ApexEmitterAssetParameters_0p8::ClassVersion>
						ConversionApexEmitterAssetParameters_0p7_0p8Parent;

class ConversionApexEmitterAssetParameters_0p7_0p8: public ConversionApexEmitterAssetParameters_0p7_0p8Parent
{
public:
	static NvParameterized::Conversion* Create(NvParameterized::Traits* t)
	{
		void* buf = t->alloc(sizeof(ConversionApexEmitterAssetParameters_0p7_0p8));
		return buf ? PX_PLACEMENT_NEW(buf, ConversionApexEmitterAssetParameters_0p7_0p8)(t) : 0;
	}

protected:
	ConversionApexEmitterAssetParameters_0p7_0p8(NvParameterized::Traits* t) : ConversionApexEmitterAssetParameters_0p7_0p8Parent(t) {}

	const NvParameterized::PrefVer* getPreferredVersions() const
	{
		static NvParameterized::PrefVer prefVers[] =
		{
			//TODO:
			//	Add your preferred versions for included references here.
			//	Entry format is
			//		{ (const char*)longName, (uint32_t)preferredVersion }

			{ 0, 0 } // Terminator (do not remove!)
		};

		return prefVers;
	}

	bool convert()
	{
		//TODO:
		//	Write custom conversion code here using mNewData and mLegacyData members.
		//
		//	Note that
		//		- mNewData has already been initialized with default values
		//		- same-named/same-typed members have already been copied
		//			from mLegacyData to mNewData
		//		- included references were moved to mNewData
		//			(and updated to preferred versions according to getPreferredVersions)
		//
		//	For more info see the versioning wiki.


		NvParameterized::Handle hOld(*mLegacyData, "iosAssetName");
		PX_ASSERT(hOld.isValid());

		NvParameterized::Handle hNew(*mNewData, "iosAssetName");
		PX_ASSERT(hNew.isValid());


		NvParameterized::Interface* oldRef = NULL;

		PARAM_RET(hOld.getParamRef(oldRef));
		PARAM_RET(hOld.setParamRef(0));
		PX_ASSERT(hOld);

		if (oldRef)
		{
			if (!physx::shdfnd::strcmp("NxBasicIosAsset", oldRef->className()))
			{
				oldRef->setClassName("BasicIosAsset");
			}
			PARAM_RET(hNew.setParamRef(oldRef));
		}

		return true;
	}
};


}
}
} //nvidia::apex::legacy

#endif
