// This code contains NVIDIA Confidential Information and is disclosed to you
// under a form of NVIDIA software license agreement provided separately to you.
//
// Notice
// NVIDIA Corporation and its licensors retain all intellectual property and
// proprietary rights in and to this software and related documentation and
// any modifications thereto. Any use, reproduction, disclosure, or
// distribution of this software and related documentation without an express
// license agreement from NVIDIA Corporation is strictly prohibited.
//
// ALL NVIDIA DESIGN SPECIFICATIONS, CODE ARE PROVIDED "AS IS.". NVIDIA MAKES
// NO WARRANTIES, EXPRESSED, IMPLIED, STATUTORY, OR OTHERWISE WITH RESPECT TO
// THE MATERIALS, AND EXPRESSLY DISCLAIMS ALL IMPLIED WARRANTIES OF NONINFRINGEMENT,
// MERCHANTABILITY, AND FITNESS FOR A PARTICULAR PURPOSE.
//
// Information and code furnished is believed to be accurate and reliable.
// However, NVIDIA Corporation assumes no responsibility for the consequences of use of such
// information or for any infringement of patents or other rights of third parties that may
// result from its use. No license is granted by implication or otherwise under any patent
// or patent rights of NVIDIA Corporation. Details are subject to change without notice.
// This code supersedes and replaces all information previously supplied.
// NVIDIA Corporation products are not authorized for use as critical
// components in life support devices or systems without express written approval of
// NVIDIA Corporation.
//
// Copyright (c) 2008-2016 NVIDIA Corporation. All rights reserved.
// Copyright (c) 2004-2008 AGEIA Technologies, Inc. All rights reserved.
// Copyright (c) 2001-2004 NovodeX AG. All rights reserved.  


#ifndef PXS_MATERIAL_H
#define PXS_MATERIAL_H

#include "foundation/PxVec3.h"
#include "PxMaterial.h"
#include "PsUserAllocated.h"
#include "CmPhysXCommon.h"
#include "PxMetaData.h"
#include "PsUtilities.h"

namespace physx
{

#define	MATERIAL_INVALID_HANDLE	0xffffffff

	//PX_ALIGN_PREFIX(16) struct PxsMaterialData 
	//{
	//	PxReal					dynamicFriction;
	//	PxReal					staticFriction;
	//	PxReal					restitution;
	//	PxReal					dynamicFrictionV;
	//	PxReal					staticFrictionV;
	//	PxVec3					dirOfAnisotropy;//might need to get rid of this
	//	PxCombineMode::Enum		frictionCombineMode;
	//	PxCombineMode::Enum		restitutionCombineMode;

	//	PxMaterialFlags			flags;
	//	PxU8					paddingFromFlags[2];
	//	PxU32					pads;

	//	PxsMaterialData()
	//	:	dynamicFriction(0.0)
	//	,	staticFriction(0.0f)
	//	,	restitution(0.0f)
	//	,	dynamicFrictionV(0.0f)
	//	,	staticFrictionV(0.0f)
	//	,	dirOfAnisotropy(1,0,0)
	//	,	frictionCombineMode(PxCombineMode::eAVERAGE)
	//	,	restitutionCombineMode(PxCombineMode::eAVERAGE)
	//	{}

	//	PX_FORCE_INLINE PxCombineMode::Enum getFrictionCombineMode() const
	//	{
	//		return frictionCombineMode;
	//	}

	//	
	//	PX_FORCE_INLINE PxCombineMode::Enum getRestitutionCombineMode() const
	//	{
	//		return restitutionCombineMode;
	//	}

	//	PX_FORCE_INLINE void setFrictionCombineMode(PxCombineMode::Enum frictionFlags)
	//	{
	//		frictionCombineMode = frictionFlags;
	//	}

	//	PX_FORCE_INLINE void setRestitutionCombineMode(PxCombineMode::Enum restitutionFlags)
	//	{
	//		restitutionCombineMode = restitutionFlags;
	//	}

	//}PX_ALIGN_SUFFIX(16);

	PX_ALIGN_PREFIX(16) struct PxsMaterialData 
	{
		PxReal					dynamicFriction;				//4
		PxReal					staticFriction;					//8
		PxReal					restitution;					//12
		PxMaterialFlags			flags;							//14
		PxU8					fricRestCombineMode;			//15
		PxU8					padding;						//16

		PxsMaterialData()
		:	dynamicFriction(0.0)
		,	staticFriction(0.0f)
		,	restitution(0.0f)
		,	fricRestCombineMode((PxCombineMode::eAVERAGE << 4) | PxCombineMode::eAVERAGE)
		{}

		PxsMaterialData(const PxEMPTY) {}

		PX_CUDA_CALLABLE PX_FORCE_INLINE PxCombineMode::Enum getFrictionCombineMode() const
		{
			return PxCombineMode::Enum(fricRestCombineMode >> 4);
		}

		PX_CUDA_CALLABLE PX_FORCE_INLINE PxCombineMode::Enum getRestitutionCombineMode() const
		{
			return PxCombineMode::Enum(fricRestCombineMode & 0xf);
		}

		PX_FORCE_INLINE void setFrictionCombineMode(PxCombineMode::Enum frictionFlags)
		{
			fricRestCombineMode = Ps::to8((fricRestCombineMode & 0xf) | (frictionFlags << 4));
		}

		PX_FORCE_INLINE void setRestitutionCombineMode(PxCombineMode::Enum restitutionFlags)
		{
			fricRestCombineMode = Ps::to8((fricRestCombineMode & 0xf0) | (restitutionFlags));
		}

	}PX_ALIGN_SUFFIX(16);


	class PxsMaterialCore : public PxsMaterialData, public Ps::UserAllocated
	{
	public:
					
											PxsMaterialCore(const PxsMaterialData& desc): PxsMaterialData(desc), mNxMaterial(0), mMaterialIndex(MATERIAL_INVALID_HANDLE)
											{
											}

											PxsMaterialCore(): mNxMaterial(0), mMaterialIndex(MATERIAL_INVALID_HANDLE)
											{
											}

											PxsMaterialCore(const PxEMPTY) : PxsMaterialData(PxEmpty) {}

											~PxsMaterialCore()
											{
											}

	PX_FORCE_INLINE	void					setNxMaterial(PxMaterial* m)					{ mNxMaterial = m;		}
	PX_FORCE_INLINE	PxMaterial*				getNxMaterial()					const			{ return mNxMaterial;	}
	PX_FORCE_INLINE	void					setMaterialIndex(const PxU32 materialIndex)		{ mMaterialIndex = materialIndex; }
	PX_FORCE_INLINE	PxU32					getMaterialIndex()				const			{ return mMaterialIndex; }

	protected:
					PxMaterial*				mNxMaterial;
					PxU32					mMaterialIndex; //handle assign by the handle manager
	};

} //namespace phyxs

#endif
